#tag Class
Protected Class StateMachineHTTPSocket
Inherits xojo.net.HTTPSocket
	#tag Event
		Function AuthenticationRequired(Realm as Text, ByRef Name as Text, ByRef Password as Text) As Boolean
		  myStatus = "StateMachineHTTPSocket Debug: AuthenticationRequired, " + realm
		  currState = "authenticationRequired"
		End Function
	#tag EndEvent

	#tag Event
		Sub Error(err as RuntimeException)
		  myStatus = "StateMachineHTTPSocket Debug: Error, " + err.Message
		  lastError = err
		  currState = "error"
		End Sub
	#tag EndEvent

	#tag Event
		Sub FileReceived(URL as Text, HTTPStatus as Integer, File as xojo.IO.FolderItem)
		  myStatus = "StateMachineHTTPSocket Debug: FileReceived, " + url + " HTTPStatus: " + format(httpStatus, "#####")
		  
		  lastHTTPStatus = HTTPStatus
		  lastFileReceived = File
		  currState = "fileReceived"
		End Sub
	#tag EndEvent

	#tag Event
		Sub HeadersReceived(URL as Text, HTTPStatus as Integer)
		  //who cares?
		  myStatus = "StateMachineHTTPSocket Debug: HeadersReceived, " + url + " HTTPStatus: " + format(httpStatus, "#####")
		End Sub
	#tag EndEvent

	#tag Event
		Sub PageReceived(URL as Text, HTTPStatus as Integer, Content as xojo.Core.MemoryBlock)
		  myStatus = "StateMachineHTTPSocket Debug: PageReceived, " + url + " HTTPStatus: " + format(httpStatus, "#####")
		  currState = "pageReceived"
		  lastHTTPStatus = httpStatus
		  lastContentReceived = Xojo.Core.TextEncoding.UTF8.ConvertDataToText(content).Trim
		  
		  myStatus = "lastContentReceived: " + lastContentReceived
		  
		End Sub
	#tag EndEvent

	#tag Event
		Sub ReceiveProgress(BytesReceived as Int64, TotalBytes as Int64, NewData as xojo.Core.MemoryBlock)
		  myStatus = "StateMachineHTTPSocket Debug: ReceiveProgress, " + " BytesReceived: " + format(BytesReceived, "############") + " Total Bytes: " + format(TotalBytes, "##########")
		  //don't care yet
		End Sub
	#tag EndEvent

	#tag Event
		Sub SendProgress(BytesSent as Int64, BytesLeft as Int64)
		  myStatus = "StateMachineHTTPSocket Debug: SendProgress, " + " BytesSent: " + format(BytesSent, "############") + " BytesLeft: " + format(BytesLeft, "##########")
		  //don't care yet
		End Sub
	#tag EndEvent


	#tag Method, Flags = &h0
		Sub Constructor()
		  // Calling the overridden superclass constructor.
		  Super.Constructor
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub prepare(mBaseURL as Text)
		  currState = "communicating"
		  lastError = nil
		  lastFileReceived = nil
		  lastHTTPStatus = 0
		  lastContentReceived = ""
		  baseURL = mBaseURL
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub setFormData(d as Dictionary)
		  dim postText as Text = ""
		  
		  //walk through the dictionary and set up key=value pairings for the data there, then cram it into the socket as the requestContent
		  //someVar=1234&schema_version=asdf
		  
		  for i as integer = 0 to d.Count - 1
		    dim key as String = d.key(i)
		    dim value as String = d.value(d.key(i))
		    postText = postText + key.toText + "=" + EncodeURLComponent(value).toText
		    if i < d.count - 1 then 
		      postText = postText + "&"
		    end if
		  next
		  
		  dim postData as xojo.Core.MemoryBlock = Xojo.core.TextEncoding.UTF8.ConvertTextToData(postText)
		  me.SetRequestContent(postData, "application/x-www-form-urlencoded")
		  
		End Sub
	#tag EndMethod


	#tag Note, Name = What is this thing?
		
		This is a subclass of xojo.net.HTTPSocket that can be used to drive a Thread-based state machine.
		
		I've been down the road of multiple different kinds of state machines (timers, threads, observers in the main thread, etc) and there are different reasons to use
		each approach.  THIS class is intended to be used by a thread-based state machine, there the thread will ask the socket to perform an operation, then the thread will put 
		itself to sleep until the socket has obtained a result.
		
		The socket tracks internal state (i.e., it keeps a hold of the last page received stuff, last error, etc.) an exposes it all so that the Thread can see what's what when it wakes up.
		
		
		currState lets us know what state this thing is in (communicating, waiting, finished, etc). This will be used to "signpost" the actions of the socket.  So when a thread is ready to 
		perform some network operation, it will set the state to commmunicating, and then call .send on this socket and check to see if the state of the socket is something other than "communicating".  
		If it is not, the thread will put itself to sleep for a little while, and check again when it wakes up.
		
		The page received, error, etc events will update the state to reflect that something has happened, so when the thread wakes up and checks, it can decide what to do next based on the 
		result of the socket operation.
		
		
	#tag EndNote


	#tag Property, Flags = &h0
		baseURL As Text
	#tag EndProperty

	#tag Property, Flags = &h0
		currState As Text
	#tag EndProperty

	#tag Property, Flags = &h0
		lastContentReceived As Text
	#tag EndProperty

	#tag Property, Flags = &h0
		lastError As RuntimeException
	#tag EndProperty

	#tag Property, Flags = &h0
		lastFileReceived As xojo.IO.FolderItem
	#tag EndProperty

	#tag Property, Flags = &h0
		lastHTTPStatus As Integer
	#tag EndProperty

	#tag Property, Flags = &h0
		myStatus As String
	#tag EndProperty


	#tag ViewBehavior
		#tag ViewProperty
			Name="baseURL"
			Group="Behavior"
			Type="Text"
		#tag EndViewProperty
		#tag ViewProperty
			Name="currState"
			Group="Behavior"
			Type="Text"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Index"
			Visible=true
			Group="ID"
			InitialValue="-2147483648"
			Type="Integer"
		#tag EndViewProperty
		#tag ViewProperty
			Name="lastContentReceived"
			Group="Behavior"
			Type="Text"
		#tag EndViewProperty
		#tag ViewProperty
			Name="lastHTTPStatus"
			Group="Behavior"
			Type="Integer"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Left"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Name"
			Visible=true
			Group="ID"
			Type="String"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Super"
			Visible=true
			Group="ID"
			Type="String"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Top"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
		#tag EndViewProperty
		#tag ViewProperty
			Name="ValidateCertificates"
			Group="Behavior"
			Type="Boolean"
		#tag EndViewProperty
	#tag EndViewBehavior
End Class
#tag EndClass
